unit CSVReaderU;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs, QMenus,
  QTypes, QImgList, QExtCtrls, QComCtrls, QStdCtrls, QButtons;

type
  TfrmMain = class(TForm)
    Images: TImageList;
    MainMenu1: TMainMenu;
    mniFile: TMenuItem;
    mniExit: TMenuItem;
    mniHelp: TMenuItem;
    mniAbout: TMenuItem;
    btnExit: TBitBtn;
    lvCDContent: TListView;
    edtName: TEdit;
    edtDescription: TEdit;
    edtURL: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    StatusBar1: TStatusBar;
    GroupBox1: TGroupBox;
    N1: TMenuItem;
    mniOpen: TMenuItem;
    dlgOpenCSV: TOpenDialog;
    lblResults: TLabel;
    procedure mniOpenClick(Sender: TObject);
    procedure mniExitClick(Sender: TObject);
    procedure mniAboutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure edtSearchFieldChange(Sender: TObject);
  private
    procedure ReadLineIntoListView(const Line: String);
    procedure LocateItems(const Name, Desc, URL: String);
    function MatchFound(const TextEntered, Value: String;
      MatchAnywhere: Boolean = False): Boolean;
  public
    procedure ReadFileIntoListView(const FileName: String);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.xfm}

const
  fldIssue       = 0;
  fldName        = 1;
  fldLocation    = 2;
  fldDescription = 3;
  fldURL         = 4;
  colName        = 0;
  colDescription = 1;
  colURL         = 2;
  colLocation    = 3;
  colIssue       = 4;

  Columns: array[colName..colIssue] of Byte =
    (fldName, fldDescription, fldURL, fldLocation, fldIssue);

procedure TfrmMain.mniOpenClick(Sender: TObject);
begin
  if dlgOpenCSV.Execute then
  begin
    lvCDContent.Items.Clear;
    ReadFileIntoListView(dlgOpenCSV.FileName);
    //Now apply search criteria from edits
    edtSearchFieldChange(nil);
  end
end;

procedure TfrmMain.mniExitClick(Sender: TObject);
begin
  Close
end;

procedure TfrmMain.mniAboutClick(Sender: TObject);
const
  About = 'Linux Format CD/DVD cover disc content browser.'#10#10 +
    'This was written for Linux Format (http://www.linuxformat.co.uk)'#10 +
    'in Borland Kylix (http://www.borland.com/Kylix)'#10 +
    'by Brian Long (http://www.blong.com)';
begin
  MessageDlg('About', About, mtInformation, [mbOk], 0)
end;

procedure TfrmMain.ReadLineIntoListView(const Line: String);
var
  LineParts: TStringList;
  Item: TListItem;
  I: Integer;
begin
  LineParts := TStringList.Create;
  try
    //Separate string parts into separate lines in TStringList
    LineParts.CommaText := Line;
    //Add text to columns, trimming off excess white space
    Item := lvCDContent.Items.Add;
    //Add in 1st column
    Item.Caption := Trim(LineParts[Columns[colName]]);
    //Add in subitems (remaining columns)
    for I := colDescription to colIssue do
      if LineParts.Count > Columns[I] then
        Item.SubItems.Add(Trim(LineParts[Columns[I]]))
      else
        Item.SubItems.Add('');
  finally
    LineParts.Free
  end;
end;

function GetCSVFileName: String;
var
  I: Integer;
  SR: TSearchRec;
begin
  for I := 1 to ParamCount do
  begin
    Result := ParamStr(I);
    if (CompareText(Result, '-ns') <> 0) and (FileExists(Result)) then
      Exit; //we found the file (Result), so leave
  end;
  Result := '';
  if FindFirst('*.csv', faAnyFile, SR) = 0 then
  begin
    Result := SR.Name;
    FindClose(SR)
  end
  else
    raise EFOpenError.Create('Cannot locate CD/DVD contents file');
end;

procedure TfrmMain.ReadFileIntoListView(const FileName: String);
var
  List: TStringList;
  I: Integer;
begin
  List := TStringList.Create;
  try
    List.LoadFromFile(FileName);
    //Process each line of file
    for I := 0 to List.Count - 1 do
      ReadLineIntoListView(List.Strings[I])
  finally
    List.Free
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  ReadFileIntoListView(GetCSVFileName);
  //Match the order in the file by default
  lvCDContent.SortColumn := colLocation;
  lvCDContent.Sorted := True;
end;

function TfrmMain.MatchFound(const TextEntered, Value: String;
  MatchAnywhere: Boolean = False): Boolean;
begin
  if MatchAnwhere then
    Result := CompareText(Copy(Value, 1, Length(TextEntered)), TextEntered) = 0
  else
    Result := Pos(UpperCase(TextEntered), UpperCase(Value)) > 0
end;

procedure TfrmMain.LocateItems(const Name, Desc, URL: String);
var
  I, Count: Integer;
  Hit: Boolean;
  Item: TListItem;
begin
  //Stop listview doing any screen updates till we are done
  lvCDContent.Items.BeginUpdate;
  Count := 0;
  for I := 0 to lvCDContent.Items.Count - 1 do
  begin
    Item := lvCDContent.Items[I];
    //Check for matching name, description or URL, if entered
    Hit := False;
    if Length(Name) > 0 then
      Hit := MatchFound(Name, Item.Caption);
    if Length(Desc) > 0 then
      Hit := Hit and MatchFound(Desc, Item.SubItems[colDescription-1]);
    if Length(URL) > 0 then
      Hit := Hit and MatchFound(URL, Item.SubItems[colURL-1]);
    if Hit then
    begin
      Inc(Count);
      Item.Selected := True;
      //If this is the first one found, focus it into view
      if Count = 1 then
        Item.Focused := True;
    end
    else
      Item.Selected := False;
  end;
  //Make listview refresh its appearance no we are done
  lvCDContent.Items.EndUpdate;
  if Count = 1 then
    lblResults.Caption := '1 item found'
  else
    lblResults.Caption := Format('%d items found', [Count])
end;

//Shared among the 3 edit controls
procedure TfrmMain.edtSearchFieldChange(Sender: TObject);
begin
  LocateItems(Trim(edtName.Text), Trim(edtDescription.Text), Trim(edtURL.Text))
end;

end.
